<?php
declare(strict_types = 1);

namespace ctr;

use lib\Db;
use mod\Account;
use mod\File;
use mod\Folder;
use mod\Mod;
use mod\Setting;
use mod\Vfile;
use sys\Ctr;

class Main extends Ctr {

    // --- 首页 ---
    public function index() {
        if (!is_file(DATA_PATH . 'mns/install.lock')) {
            // --- 是否要安装 ---
            return $this->_location(URL_BASE . 'install');
        }
        // --- 已安装 ---
        $db = Db::get();
        if (!$db->connect()) {
            return 'Site has an error.';
        }
        Mod::setDb($db);
        if (PRODUCT === 'zuny') {
            // --- ZUNY ---
            $setting = Setting::where([
                'name' => ['open', 'closeinfo', 'title', 'footer', 'logo', 'lang', 'keywords', 'description', 'introduce']
            ])->all('name');
            $open = (int)$setting['open']->value;
            // --- 加载语言 ---
            $this->_loadLocale($setting['lang']->value);
            // --- 当前地址 ---
            $parent = isset($this->_param[0]) ? $this->_param[0] : '';
            // --- 加载项目下的内容 ---
            $rtn = null;
            if ($open === 1) {
                $rtn = Folder::getAllList($parent, '1', '0');
                if (!$rtn) {
                    return $this->_location(URL_BASE);
                }
            }
            $this->_enabledXsrf();
            return $this->_loadView('zuny/folder', [
                'title' => ($rtn && $rtn['name'] ? $rtn['name'] . ' - ' : '') . $setting['title']->value,
                'footer' => $setting['footer']->value,
                'logo' => $setting['logo']->value,
                'introduce' => $setting['introduce']->value,

                'parent' => $parent,
                'path' => $rtn ? $rtn['path'] : [],
                'pathKey' => $rtn ? $rtn['pathKey'] : [],
                'list' => $rtn ? $rtn['list'] : [],
                'open' => $open,
                'closeinfo' => $setting['closeinfo']->value,
                'keywords' => $setting['keywords']->value,
                'description' => $setting['description']->value,

                'xsrf' => $this->_xsrf
            ]);
        }
        return 'Site has an error, page not found.';
    }

    // --- 文件页面（或直接下载） ---
    public function file() {
        if (PRODUCT === 'zuny') {
            // --- ZUNY ---
            if (EDITION_NUM <= EDITION_TRIAL_NUM) {
                return $this->_location(URL_BASE . 'zuny/trial/file?key=' . $this->_param[0]);
            }
            else {
                return $this->_location(URL_BASE . 'zuny/std/file?key=' . $this->_param[0]);
            }
        }
        else if (PRODUCT === 'filiant') {
            // --- FILIANT ---
        }
        return $this->_location(URL_BASE);
    }

    // --- 直接下载 ---
    public function down() {
        if (PRODUCT === 'zuny') {
            // --- ZUNY ---
            if (EDITION_NUM <= EDITION_TRIAL_NUM) {
                return $this->_location(URL_BASE . 'zuny/trial/down?key=' . $this->_param[0]);
            }
            else {
                return $this->_location(URL_BASE . 'zuny/std/down?key=' . $this->_param[0]);
            }
        }
        else if (PRODUCT === 'filiant') {
            // --- FILIANT ---
        }
        return $this->_location(URL_BASE);
    }

    // --- 后台 ---
    public function panel() {
        $db = Db::get();
        if (!$db->connect()) {
            return 'Site has an error.';
        }
        Mod::setDb($db);
        $this->_startSession($db);
        $acc = Account::getLogined();
        if (!$acc) {
            return $this->_location(URL_BASE);
        }
        if (!$acc->checkPermission('root')) {
            return $this->_location(URL_BASE);
        }
        $setting = Setting::where([
            'name' => ['lang']
        ])->all('name');
        // --- 加载语言 ---
        $this->_loadLocale($setting['lang']->value);
        $this->_enabledXsrf();
        return $this->_loadView('panel', [
            'lang' => $setting['lang']->value,
            'xsrf' => $this->_xsrf
        ]);
    }

}

