<?php
declare(strict_types = 1);

namespace ctr;

use lib\Core;
use lib\Db;
use lib\Net;
use lib\Text;
use mod\Account;
use mod\File;
use mod\Mod;
use mod\Setting;
use mod\Vfile;
use PHPMailer\PHPMailer\Exception;
use PHPMailer\PHPMailer\PHPMailer;
use sys\Ctr;

class Panel extends Ctr {

    public function onLoad() {
        $db = Db::get();
        if (!$db->connect()) {
            return 'Site has an error.';
        }
        Mod::setDb($db);
        $this->_startSession($db);
        $setting = Setting::where([
            'name' => ['lang']
        ])->all('name');
        $this->_loadLocale($setting['lang']->value);
    }

    public function getIndex() {
        $acc = Account::getLogined();
        if (!$acc) {
            return [0, l('please-log-in-first')];
        }
        if (!$acc->checkPermission('root')) {
            return [0, l('no-permission')];
        }
        return [1,
            'info' => [
                'product' => PRODUCT_W,
                'pversion' => PVERSION,
                'edition' => ucwords(EDITION)
            ]
        ];
    }

    public function getServerInfo() {
        $acc = Account::getLogined();
        if (!$acc) {
            return [0, l('please-log-in-first')];
        }
        if (!$acc->checkPermission('root')) {
            return [0, l('no-permission')];
        }
        return [1,
            'info' => [
                'serverSoftware' => $_SERVER['SERVER_SOFTWARE'],
                'postMaxSize' => ini_get('post_max_size'),
                'uploadMaxFilesize' => ini_get('upload_max_filesize'),
                'monacoCheck' => is_dir(ROOT_PATH . 'stc/cdn/npm/monaco-editor@0.34.1/'),
                'phpmailerCheck' => is_file(ROOT_PATH . 'lib/PHPMailer-6.7.1/get_oauth_token.php'),
                'memoryLimit' => ini_get('memory_limit'),
                'diskFreeSpace' => Text::sizeFormat(@disk_free_space('.')),
                'memoryUsage' => Text::sizeFormat(memory_get_usage())
            ]
        ];
    }

    public function getSetting() {
        $acc = Account::getLogined();
        if (!$acc) {
            return [0, l('please-log-in-first')];
        }
        if (!$acc->checkPermission('root')) {
            return [0, l('no-permission')];
        }
        if (!$this->_checkInput($this->_post, [
            'name' => []
        ], $rtn)) {
            return $rtn;
        }

        $setting = [];
        $ls = Setting::where([
            'name' => explode(',', $this->_post['name'])
        ])->all();
        foreach ($ls as $item) {
            $setting[$item->name] = $item->value;
        }

        return [1,
            'setting' => $setting
        ];
    }

    public function saveSetting() {
        $acc = Account::getLogined();
        if (!$acc) {
            return [0, l('please-log-in-first')];
        }
        if (!$acc->checkPermission('root')) {
            return [0, l('no-permission')];
        }
        if (!$this->_checkXInput($this->_post, [
            'setting' => ['require', [0, 'Param error.']]
        ], $rtn)) {
            return $rtn;
        }
        foreach ($this->_post['setting'] as $name => $val) {
            Setting::updateByWhere([
                'value' => $val
            ], [
                'name' => $name
            ]);
        }
        return [1];
    }

    public function upload() {
        $acc = Account::getLogined();
        if (!$acc) {
            return [0, l('please-log-in-first')];
        }
        if (!$acc->checkPermission('root')) {
            return [0, l('no-permission')];
        }
        if (!$this->_checkXInput($this->_post, [
            'type' => ['require', ['logo'], [0, 'Param error.']]
        ], $rtn)) {
            return $rtn;
        }
        if (!isset($_FILES['file'])) {
            return [0, 'Param error.'];
        }
        $sp = strrpos($_FILES['file']['name'], '.');
        if ($sp === false) {
            return [0, l('file format not supported')];
        }
        $ext = strtolower(substr($_FILES['file']['name'], $sp + 1));
        if (!in_array($ext, ['png', 'jpg', 'svg', 'gif', 'jpeg'])) {
            return [0, l('file format not supported') . '(' . $ext . ')'];
        }
        if (!move_uploaded_file($_FILES['file']['tmp_name'], ROOT_PATH . 'stc/files/logo.' . $ext)) {
            return [0, l('upload-failed')];
        }
        return [1, 'logo' => 'logo.' . $ext];
    }

    // --- 发信测试 ---
    public function smtpTest() {
        $acc = Account::getLogined();
        if (!$acc) {
            return [0, l('please-log-in-first')];
        }
        if (!$acc->checkPermission('root')) {
            return [0, l('no-permission')];
        }
        if (!$this->_checkXInput($this->_post, [
            'setting' => ['require', [0, 'Param error(1).']],
            'receive_mailbox' => ['require', [0, l('required-field-is-missing')]],
            'mail_subject' => ['require', [0, l('required-field-is-missing')]],
            'mail_content' => ['require', [0, l('required-field-is-missing')]]
        ], $rtn)) {
            return $rtn;
        }

        if (!is_file(LIB_PATH . 'PHPMailer-6.7.1/src/Exception.php')) {
            return [0, l('the relevant library does not exist and cannot send a message')];
        }
        try {
            require LIB_PATH . 'PHPMailer-6.7.1/src/Exception.php';
            require LIB_PATH . 'PHPMailer-6.7.1/src/PHPMailer.php';
            require LIB_PATH . 'PHPMailer-6.7.1/src/SMTP.php';
            $mail = new PHPMailer(true);
            $mail->isSMTP();
            $mail->SMTPAuth = true;
            $mail->CharSet = 'utf-8';
            $mail->Host = $this->_post['setting']['smtp_server'];
            $mail->Port = $this->_post['setting']['smtp_port'];
            $mail->Username = $this->_post['setting']['smtp_user'];
            $mail->Password = $this->_post['setting']['smtp_pwd'];
            if ($this->_post['setting']['smtp_ssl'] === '1') {
                $mail->SMTPSecure = PHPMailer::ENCRYPTION_SMTPS;
            }
            $mail->setFrom($this->_post['setting']['smtp_from'], $this->_post['setting']['smtp_name']);

            $mail->addAddress($this->_post['receive_mailbox']);
            $mail->isHTML(true);
            $mail->Subject = "=?utf-8?B?" . base64_encode($this->_post['mail_subject']) . "?=";
            $mail->Body = $this->_post['mail_content'];

            $mail->send();

            return [1];
        }
        catch (Exception $e) {
            return [0, $mail->ErrorInfo];
        }
    }

    // --- 获取用户列表 ---
    public function getUserList() {
        $acc = Account::getLogined();
        if (!$acc) {
            return [0, l('please-log-in-first')];
        }
        if (!$acc->checkPermission('root')) {
            return [0, l('no-permission')];
        }
        if (!$this->_checkInput($this->_post, [
            'user' => [],
            'email' => [],
            'phone' => [],
            'page' => ['require', 'num', [0, l('required-field-is-missing')]]
        ], $rtn)) {
            return $rtn;
        }
        $ls = Account::where()->page(20, $this->_post['page']);
        $lss = $ls->all();
        $list = [];
        foreach ($lss as $item) {
            $list[] = [
                'id' => (int)$item->id,
                'name' => $item->name,
                'email' => $item->email,
                'phone' => $item->area . ' ' . $item->phone,
                'permission' => $item->permission,
                'disk_usage' => Text::sizeFormat($item->disk_usage),
                'disk_max' => Text::sizeFormat($item->disk_max),
                'disk_remain' => Text::sizeFormat($item->disk_remain),
                'files' => $item->files,
                'folders' => $item->folders,
                'ban' => (int)$item->ban,
                'time_add' => date('Y-m-d H:i:s', (int)$item->time_add)
            ];
        }
        return [1, 'list' => $list, 'total' => $ls->total()];
    }

    // --- 获取文件列表 ---
    public function getFileList() {
        $acc = Account::getLogined();
        if (!$acc) {
            return [0, l('please-log-in-first')];
        }
        if (!$acc->checkPermission('root')) {
            return [0, l('no-permission')];
        }
        if (!$this->_checkInput($this->_post, [
            'ext' => [],
            'index' => ['require', 'num', [0, l('required-field-is-missing')]],
            'page' => ['require', 'num', [0, l('required-field-is-missing')]]
        ], $rtn)) {
            return $rtn;
        }
        // --- 判断分表 ---
        $setting = Setting::where([
            'name' => ['fsplit']
        ])->all('name');
        if ((int)$this->_post['index'] >= (int)$setting['fsplit']->value) {
            return [0, l('split table does not exist')];
        }
        // --- where ---
        $where = [];
        if ($this->_post['ext']) {
            $where['file_' . $this->_post['index'] . '.ext'] = $this->_post['ext'];
        }
        $ls = File::select(['file_' . $this->_post['index'] . '.*', 'ac1.name owner_name', 'ac2.name creator_name'], $this->_post['index'])->filter($where, false, $this->_post['index'])->leftJoin('account ac1', [
            'ac1.id' => '#file_' . $this->_post['index'] . '.owner'
        ])->leftJoin('account ac2', [
            'ac2.id' => '#file_' . $this->_post['index'] . '.creator'
        ])->page(20, $this->_post['page']);
        $lss = $ls->all();
        $list = [];
        foreach ($lss as $item) {
            $list[] = [
                'id' => (int)$item->id,
                'owner' => $item->owner,
                'owner_name' => $item->owner_name,
                'creator' => $item->creator,
                'creator_name' => $item->creator_name,
                'name' => $item->name,
                'size' => Text::sizeFormat($item->size),
                'dv' => $item->down . '/' . $item->visit,
                'url' => $item->url,
                'time_add' => date('Y-m-d H:i:s', (int)$item->time_add),
                'time_edit' => date('Y-m-d H:i:s', (int)$item->time_edit)
            ];
        }
        return [1, 'list' => $list, 'total' => $ls->total()];
    }

    // --- 获取V文件列表 ---
    public function getVfileList() {
        $acc = Account::getLogined();
        if (!$acc) {
            return [0, l('please-log-in-first')];
        }
        if (!$acc->checkPermission('root')) {
            return [0, l('no-permission')];
        }
        if (!$this->_checkInput($this->_post, [
            'noreference' => ['require', 'bool', [0, l('required-field-is-missing')]],
            'index' => ['require', 'num', [0, l('required-field-is-missing')]],
            'page' => ['require', 'num', [0, l('required-field-is-missing')]]
        ], $rtn)) {
            return $rtn;
        }
        // --- 判断分表 ---
        $setting = Setting::where([
            'name' => ['vsplit']
        ])->all('name');
        if ((int)$this->_post['index'] >= (int)$setting['vsplit']->value) {
            return [0, l('split table does not exist')];
        }
        // --- where ---
        $where = [];
        if ($this->_post['noreference']) {
            $where['ref'] = '0';
        }
        else {
            $where[] = ['ref', '>', '0'];
        }
        $ls = Vfile::where($where, false, $this->_post['index'])->page(20, $this->_post['page']);
        $lss = $ls->all();
        $list = [];
        foreach ($lss as $item) {
            $list[] = [
                'id' => (int)$item->id,
                'key' => $item->key,
                'size' => Text::sizeFormat($item->size),
                'sha256' => $item->sha256 ? $item->sha256 : l('none'),
                'server' => $item->server,
                'path' => $item->path,
                'ref' => $item->ref,
                'time_add' => date('Y-m-d H:i:s', (int)$item->time_add)
            ];
        }
        return [1, 'list' => $list, 'total' => $ls->total()];
    }

    // --- 获取用户 ---
    public function getUser() {
        $acc = Account::getLogined();
        if (!$acc) {
            return [0, l('please-log-in-first')];
        }
        if (!$acc->checkPermission('root')) {
            return [0, l('no-permission')];
        }
        if (!$this->_checkInput($this->_post, [
            'id' => ['require', 'num', [0, l('param-error')]]
        ], $rtn)) {
            return $rtn;
        }
        $account = Account::find($this->_post['id']);
        if (!$account) {
            return [0, l('account-does-not-exist')];
        }
        return [1,
            'name' => $account->name,
            'email' => $account->email,
            'area' => $account->area,
            'phone' => $account->phone
        ];
    }

    // --- 获取用户权限 ---
    public function getPermission() {
        $acc = Account::getLogined();
        if (!$acc) {
            return [0, l('please-log-in-first')];
        }
        if (!$acc->checkPermission('root')) {
            return [0, l('no-permission')];
        }
        if (!$this->_checkInput($this->_post, [
            'id' => ['require', 'num', [0, l('param-error')]]
        ], $rtn)) {
            return $rtn;
        }
        $account = Account::find($this->_post['id']);
        if (!$account) {
            return [0, l('account-does-not-exist')];
        }
        return [1,
            'permission' => $account->permission
        ];
    }

    // --- 删除文件 ---
    public function deleteFile() {
        $acc = Account::getLogined();
        if (!$acc) {
            return [0, l('please-log-in-first')];
        }
        if (!$acc->checkPermission('root')) {
            return [0, l('no-permission')];
        }
        if (!$this->_checkXInput($this->_post, [
            'id' => ['require', 'num', [0, l('param-error')]],
            'index' => ['require', 'num', [0, l('param-error')]]
        ], $rtn)) {
            return $rtn;
        }
        // --- 判断分表 ---
        $setting = Setting::where([
            'name' => ['fsplit']
        ])->all('name');
        if ((int)$this->_post['index'] >= (int)$setting['fsplit']->value) {
            return [0, l('split table does not exist')];
        }
        $file = File::one([
            'id' => $this->_post['id']
        ], false, $this->_post['index']);
        if (!$file) {
            return [0, l('file-not-exist')];
        }
        $setting = Setting::where([
            'name' => ['vsplit']
        ])->all('name');
        $file->delete((int)$setting['vsplit']->value);
        return [1];
    }

    // --- 删除V文件 ---
    public function deleteVfile() {
        $acc = Account::getLogined();
        if (!$acc) {
            return [0, l('please-log-in-first')];
        }
        if (!$acc->checkPermission('root')) {
            return [0, l('no-permission')];
        }
        if (!$this->_checkXInput($this->_post, [
            'id' => ['require', 'num', [0, l('param-error')]],
            'index' => ['require', 'num', [0, l('param-error')]]
        ], $rtn)) {
            return $rtn;
        }
        // --- 判断分表 ---
        $setting = Setting::where([
            'name' => ['vsplit']
        ])->all('name');
        if ((int)$this->_post['index'] >= (int)$setting['vsplit']->value) {
            return [0, l('split table does not exist')];
        }
        $vfile = Vfile::one([
            'id' => $this->_post['id']
        ], false, $this->_post['index']);
        if (!$vfile) {
            return [0, l('file-not-exist')];
        }
        if ($vfile->ref > 0) {
            return [0, l('the current file has references that cannot be deleted')];
        }
        $vfile->delete();
        return [1];
    }

    // --- 保存用户 ---
    public function saveUser() {
        $acc = Account::getLogined();
        if (!$acc) {
            return [0, l('please-log-in-first')];
        }
        if (!$acc->checkPermission('root')) {
            return [0, l('no-permission')];
        }
        if (!$this->_checkXInput($this->_post, [
            'id' => ['require', 'num', [0, l('param-error')]],
            'name' => ['require', [0, l('required-field-is-missing')]],
            'password' => [],
            'email' => ['require', [0, l('required-field-is-missing')]],
            'area' => ['require', 'num', [0, l('required-field-is-missing')]],
            'phone' => ['require', 'num', [0, l('required-field-is-missing')]],
            'disk_max' => ['require', 'num', [0, l('required-field-is-missing')]]
        ], $rtn)) {
            return $rtn;
        }

        $passowrd = '';
        $salt = '';
        if ($this->_post['password']) {
            $salt = Core::random(4);
            $passowrd = hash_hmac('md5', $this->_post['password'], $salt);
        }

        $id = (int)$this->_post['id'];
        if ($id) {
            // --- 保存 ---
            $set = [
                'name' => $this->_post['name'],
                'email' => $this->_post['email'],
                'area' => $this->_post['area'],
                'phone' => $this->_post['phone'],
                'time_add' => time()
            ];
            if ($passowrd) {
                $set['password'] = $passowrd;
                $set['salt'] = $salt;
            }
            Account::updateByWhere($set, [
                'id' => $id
            ]);
        }
        else {
            // --- 创建 ---
            if (!$passowrd) {
                return [0, l('required-field-is-missing')];
            }
            $account = Account::getCreate();
            $account->set([
                'name' => $this->_post['name'],
                'password' => $passowrd,
                'salt' => $salt,
                'email' => $this->_post['email'],
                'area' => $this->_post['area'],
                'phone' => $this->_post['phone'],
                'permission' => '',
                'disk_usage' => '0',
                'disk_max' => $this->_post['disk_max'],
                'disk_remain' => $this->_post['disk_max'],
                'files' => '0',
                'folders' => '0',
                'ban' => '0',
                'time_add' => time()
            ]);
            $account->create();
        }
        return [1];
    }

    // --- 保存用户权限 ---
    public function savePermission() {
        $acc = Account::getLogined();
        if (!$acc) {
            return [0, l('please-log-in-first')];
        }
        if (!$acc->checkPermission('root')) {
            return [0, l('no-permission')];
        }
        if (!$this->_checkXInput($this->_post, [
            'id' => ['require', 'num', [0, l('param-error')]],
            'permission' => []
        ], $rtn)) {
            return $rtn;
        }
        Account::updateByWhere([
            'permission' => $this->_post['permission']
        ], [
            'id' => $this->_post['id']
        ]);
        return [1];
    }

    // --- 禁用，启用用户 ---

    public function banUser() {
        $acc = Account::getLogined();
        if (!$acc) {
            return [0, l('please-log-in-first')];
        }
        if (!$acc->checkPermission('root')) {
            return [0, l('no-permission')];
        }
        if (!$this->_checkXInput($this->_post, [
            'id' => ['require', 'num', [0, l('param-error')]],
            'ban' => ['require', 'num', [0, l('param-error')]]
        ], $rtn)) {
            return $rtn;
        }
        if ((int)$this->_post['id'] == 1) {
            return [0, l('users with id 1 cannot be disabled')];
        }
        Account::updateByWhere([
            'ban' => $this->_post['ban']
        ], [
            'id' => $this->_post['id']
        ]);
        return [1];
    }

}

