<?php
declare(strict_types = 1);

namespace ctr\zuny;

use lib\Db;
use mod\File;
use mod\Mod;
use mod\Setting;
use mod\Vfile;
use sys\Ctr;

class Trial extends Ctr {

    public function onLoad() {
        $db = Db::get();
        if (!$db->connect()) {
            return 'Site has an error.';
        }
        Mod::setDb($db);
    }

    // --- 文件页面 ---
    public function file() {
        $setting = Setting::where([
            'name' => ['open']
        ])->all('name');
        $open = (int)$setting['open']->value;
        if ($open === 0) {
            // --- 站点关闭，跳转到首页 ---
            return $this->_location(URL_BASE);
        }
        // --- 跳转到直接下载 ---
        return $this->_location(URL_BASE . 'zuny/trial/down?key=' . $this->_get['key']);
    }

    // --- 文件下载 ---
    public function down() {
        $setting = Setting::where([
            'name' => ['open', 'fsplit', 'vsplit']
        ])->all('name');
        $open = (int)$setting['open']->value;
        if ($open === 0) {
            // --- 站点关闭，跳转到首页 ---
            return $this->_location(URL_BASE);
        }
        // --- 判断分表 ---
        $findex = '0';
        if ($setting['fsplit']->value > 1) {
            // --- 分表 ---
        }
        $vindex = '0';
        if ($setting['vsplit']->value > 1) {
            // --- 分表 ---
        }
        $file = File::one([
            'key' => $this->_get['key']
        ], false, $findex);
        if (!$file) {
            // --- 文件不存在 ---
            return $this->_location(URL_BASE);
        }
        $vfile = Vfile::one([
            'key' => $file->vfile
        ], false, $vindex);
        if (!$vfile) {
            // --- 文件不存在 ---
            return $this->_location(URL_BASE);
        }
        /** --- 本地真实路径 --- */
        $real = DATA_PATH . 'files/' . $vfile->path;
        // --- 是否断点 ---
		define('IS_HTTPRANGE', isset($_SERVER['HTTP_RANGE']) && ($_SERVER['HTTP_RANGE'] !== '') ? true : false);
        // --- 开始输出 ---
		@set_time_limit(0);
		header('content-type: application/octet-stream');
		header('accept-ranges: bytes');
        $fh = fopen($real, 'r');
		// --- 设置文件名 ---
		$_SERVER['HTTP_USER_AGENT'] = strtolower($_SERVER['HTTP_USER_AGENT']);
		if (preg_match('/msie/', $_SERVER['HTTP_USER_AGENT']) || preg_match("/trident\/7.0/", $_SERVER['HTTP_USER_AGENT'])) {
            $filenameEncoded = str_replace('+', '%20', urlencode($file->name));
			header('content-disposition: attachment; filename="' . $filenameEncoded . '"');
        }
		else if (preg_match('/firefox/', $_SERVER['HTTP_USER_AGENT'])) {
			header('content-disposition: attachment; filename*="utf8\'\'' . $file->name . '"');
		}
        else {
			header('content-disposition: attachment; filename="' . $file->name . '"');
        }
		// --- 检测断点 ---
		if (IS_HTTPRANGE) {
			// --- 是断点 ---
			list($httpRangeMin, $httpRangeMax) = explode('-', str_replace('bytes=', '', trim(strtolower($_SERVER['HTTP_RANGE']))));
            $httpRangeMin = (int)$httpRangeMin;
			if ($httpRangeMin === 0) {
				// --- 从 0 开始的断点就不是断点 ---
			}
            else {
                fseek($fh, $httpRangeMin);
				header('content-length: ' . ($vfile->size - 1 - $httpRangeMin));
				header('content-range: bytes ' . $httpRangeMin . '-' . ($vfile->size - 1) . '/' . $vfile->size);
				header('HTTP/1.1 206 Partial Content');
			}
		}
        else {
			// --- 没断点 ---
			header('content-range: bytes 0-' . ($vfile->size - 1) . '/' . $vfile->size);
			header('content-length: ' . $vfile->size);
		}
		// --- Start Download ---
        while (!feof($fh)) {
            echo fread($fh, 32768);
            ob_flush();
            flush();
        }
        fclose($fh);
    }
}

