<?php
declare(strict_types = 1);

namespace mod;

class Account extends Mod {

    protected static $_table = 'account';
    protected static $_primary = 'id';

    public $id, $name, $password, $salt, $email, $area, $phone, $permission, $disk_usage, $disk_max, $disk_remain, $files, $folders, $ban, $time_add;

    /** --- 获取当前已登录用户 --- */
    public static function getLogined(): Account | null {
        if (!isset($_SESSION['user'])) {
            return null;
        }
        $acc = self::find($_SESSION['user']['id']);
        if (!$acc) {
            unset($_SESSION['user']);
            return null;
        }
        if ($acc->salt !== $_SESSION['user']['salt']) {
            unset($_SESSION['user']);
            return null;
        }
        if ((int)$acc->ban) {
            // --- 禁用的用户 ---
            unset($_SESSION['user']);
            return null;
        }
        return $acc;
    }

    /** --- 检测用户权限 --- */
    public function checkPermission(string $val) {
        if (strpos(',' . $this->permission . ',', ',' . $val . ',') === false) {
            return false;
        }
        return true;
    }

    /** --- 申请空间 --- */
    public function requestSpace(int $space): bool {
        return Account::updateByWhere([
            ['disk_usage', '+', $space],
            ['disk_remain', '-', $space]
        ], [
            'id' => $this->id,
            ['disk_remain', '>=', $space]
        ]) ? true : false;
    }

    /** --- 释放空间（删除文件、上传失败等情况） --- */
    public function restoreSpace(int $space): void {
        Account::updateByWhere([
            ['disk_usage', '-', $space],
            ['disk_remain', '+', $space]
        ], [
            'id' => $this->id
        ]);
    }

}

