<?php
declare(strict_types = 1);

namespace mod;

use lib\Consistent;

class File extends Mod {

    protected static $_table = 'file';
    protected static $_primary = 'id';

    public $id, $key, $owner, $creator, $parent, $name, $ext, $size, $path, $path_key, $down, $visit, $vfile, $url, $time_add, $time_edit;

    /**
     * --- 将文件夹更名为新名 ---
     * @param $name 新名
     */
    public function rename(string $name): bool {
        if (!$name) {
            return false;
        }
        if (Folder::checkDuplicate($name, $this->parent, $this->owner, $this->_index)) {
            return false;
        }
        $fsp = strrpos($name, '.');
        $ext = $fsp === false ? '' : substr($name, $fsp + 1);
        if (!File::updateByWhere([
            'name' => $name,
            'ext' => $ext,
            'time_edit' => time()
        ], [
            'key' => $this->key
        ], false, $this->_index)) {
            return false;
        }
        $this->name = $name;
        return true;
    }

    /**
     * --- 删除当前文件（不处理 vfile） ---
     */
    public function delete($vHashCount = 1, $raw = false): int {
        if ($this->parent) {
            // --- 循环向上减 ---
            $parent = $this->parent;
            while ($parent) {
                $pf = Folder::one([
                    'key' => $parent
                ], false, $this->_index);
                Folder::updateByWhere([
                    ['files', '-', 1],
                    ['size', '-', $this->size]
                ], [
                    'id' => $pf->id
                ], false, $this->_index);
                if ($pf->parent === '') {
                    break;
                }
                $parent = $pf->parent;
            }
        }
        $this->remove($raw);
        Account::updateByWhere([
            ['files', '-', '1'],
            ['disk_usage', '-', $this->size],
            ['disk_remain', '+', $this->size]
        ], [
            'id' => $this->owner
        ]);
        if ($this->vfile) {
            $index = '0';
            if ($vHashCount > 1) {
                $nodes = Consistent::getRange(0, $vHashCount - 1);
                $index = Consistent::fast($this->vfile, $nodes);
            }
            Vfile::updateByWhere([
                ['ref', '-', '1']
            ], [
                'key' => $this->vfile
            ], false, $index);
        }
        return $this->size;
    }

    /**
     * --- 移动一个文件到一个新位置 ---
     */
    public static function moveFile($key, $parent, $index): int {
        $file = File::one([
            'key' => $key
        ], false, $index);
        if (!$file) {
            return 0;
        }
        if ($file->parent == $parent) {
            return 1;
        }
        if (Folder::checkDuplicate($file->name, $parent, $file->owner, $index)) {
            return -1;
        }
        $newPath = '';
        $newPathKey = '';
        if ($parent) {
            // --- $parent 是要移动到的目标文件夹，有可能为空，空代表的顶级 ---
            $pf = Folder::one([
                'key' => $parent
            ], false, $index);
            if (!$pf) {
                return 0;
            }
            if ($pf->path) {
                $newPath = $pf->path . '/';
                $newPathKey = $pf->path_key . '/';
            }
            $newPath .= $pf->name;
            $newPathKey .= $pf->key;
        }
        // --- 移动 ---
        if (!File::updateByWhere([
            'path' => $newPath,
            'path_key' => $newPathKey,
            'parent' => $parent
        ], [
            'id' => $file->id
        ], false, $index)) {
            return 0;
        }
        // --- 给老的 parent 减少 size、files 等计数 ---
        $oldParent = $file->parent;
        if ($oldParent) {
            while ($oldParent) {
                $pf = Folder::one([
                    'key' => $oldParent
                ], false, $index);
                Folder::updateByWhere([
                    ['files', '-', 1],
                    ['size', '-', $file->size]
                ], [
                    'id' => $pf->id
                ], false, $index);
                if ($pf->parent === '') {
                    break;
                }
                $oldParent = $pf->parent;
            }
        }
        // --- 给新的增加上面说的那些 ---
        if ($parent) {
            while ($parent) {
                $pf = Folder::one([
                    'key' => $parent
                ], false, $index);
                Folder::updateByWhere([
                    ['files', '+', 1],
                    ['size', '+', $file->size]
                ], [
                    'id' => $pf->id
                ], false, $index);
                if ($pf->parent === '') {
                    break;
                }
                $parent = $pf->parent;
            }
        }
        return 1;
    }

}

