import * as tool from '../../tool';

let vroot: IVue;

const xhr = new XMLHttpRequest();
xhr.upload.onprogress = function(e: ProgressEvent): void {
    vroot.upFiles[0].per = Math.round((e.loaded / e.total) * 100).toString();
};
xhr.onload = function(): void {
    try {
        const j = JSON.parse(xhr.response);
        if (j.result <= 0) {
            alert(j.msg);
            vroot.upFiles.splice(0, 1);
            vroot.uploadHandler();
            return;
        }
        vroot.refresh();
        vroot.upFiles.splice(0, 1);
        vroot.uploadHandler();
    }
    catch {
        vroot.upFiles.splice(0, 1);
        vroot.uploadHandler();
    }
};
xhr.onerror = function(): void {
    vroot.upFiles.splice(0, 1);
    vroot.uploadHandler();
};

const globalConst: {
    'urlBase': string;
    'xsrf': string;
    'parent': string;
    'path': any[];
    'pathKey': any[];
    'list': any[];
} = (window as any).globalConst;
const app = Vue.createApp({
    'data': function() {
        return {
            'mask': false,
            'maskt': false,
            'rightMenuShow': false,
            'itemMenuShow': false,
            'dialogType': '',
            'folderName': '',
            'folderNote': '',
            'dialogTxt': '',
            'stype': '',
            'skey': '',
            'fileDrag': false,
            'files': [],
            'upFiles': [],
            // --- 移动相关 ---
            'mlist': [],
            'mpath': [],
            'mpathKey': [],
            /** --- 当前已登录用户名 --- */
            'account': '',
            'user': '',
            'password': '',
            'captcha': '',
            'captchaImg': '',
            // --- 界面相关 ---
            'list': [],
            'path': [],
            'pathKey': []
        };
    },
    'methods': {
        // --- menu ---
        openMenu: async function(this: IVue, e: MouseEvent) {
            if (this.rightMenuShow) {
                return;
            }
            if (this.account === '') {
                this.mask = true;
                const r = await fetch(globalConst.urlBase + 'api/zuny/checkLogin');
                const j = await r.json();
                this.account = j.account;
                this.mask = false;
            }
            const rect = this.$refs.add.getBoundingClientRect();
            this.$refs.rightMenu.style.right = window.innerWidth - rect.x - rect.width + 'px';
            this.$refs.rightMenu.style.top = (rect.y + rect.height + 10) + 'px'
            this.rightMenuShow = true;
        },
        hideMenu: function(this: IVue) {
            if (this.rightMenuShow) {
                this.rightMenuShow = false;
            }
            if (this.itemMenuShow) {
                this.itemMenuShow = false;
            }
        },
        // --- menu 创建文件夹 ---
        createFolder: function(this: IVue) {
            if (!this.account) {
                this.login();
                return;
            }
            this.dialogType = 'folder';
            this.folderName = '';
            this.folderNote = '';
            this.hideMenu();
        },
        // --- 打开上传窗口 ---
        upload: function(this: IVue) {
            if (!this.account) {
                this.login();
                return;
            }
            this.dialogType = 'upload';
            this.hideMenu();
            this.files.length = 0;
        },
        // --- 上传窗口的拖拽事件 ---
        fileDragEnter: function(this: IVue, e: DragEvent) {
            e.preventDefault();
            this.fileDrag = true;
        },
        fileDragLeave: function(this: IVue, e: DragEvent) {
            e.preventDefault();
            this.fileDrag = false;
        },
        fileDrop: function(this: IVue, e: DragEvent) {
            e.preventDefault();
            this.fileDrag = false;
            if (!e.dataTransfer?.files) {
                return;
            }
            for (const file of e.dataTransfer.files) {
                if (this.files.length >= 10) {
                    break;
                }
                this.files.push(file);
            }
        },
        fileChange: function(this: IVue) {
            if (!this.$refs.file.files.length) {
                return;
            }
            const files = this.$refs.file.files;
            for (let i = 0; i < files.length; ++i) {
                if (this.files.length >= 10) {
                    break;
                }
                this.files.push(files[i]);
            }
            this.$refs.file.value = '';
        },
        toUpload: function(this: IVue) {
            this.dialogType = '';
            for (const item of this.files) {
                this.upFiles.push({
                    'name': item.name,
                    'file': item,
                    'per': '0'
                });
            }
            if (this.upFiles.length === this.files.length) {
                this.uploadHandler();
            }
        },
        uploadHandler: function(this: IVue) {
            if (!this.upFiles.length) {
                return;
            }
            // --- 创建上传事件 ---
            const fd = new FormData();
            fd.append('parent', globalConst.parent);
            fd.append('file', this.upFiles[0].file);
            fd.append('_xsrf', globalConst.xsrf);
            xhr.open('POST', globalConst.urlBase + 'api/zuny/upload', true);
            xhr.send(fd);
        },
        // --- 创建文件夹的确定按钮 ---
        create: async function(this: IVue) {
            this.mask = true;
            this.dialogType = '';
            const r = await tool.post(globalConst.urlBase + 'api/zuny/createFolder', {
                'name': this.folderName,
                'note': this.folderNote,
                'parent': globalConst.parent,
                '_xsrf': globalConst.xsrf
            });
            const j = await r.json();
            if (j.result <= 0) {
                this.mask = false;
                this.dialogType = 'dialog';
                this.dialogTxt = j.msg;
                return;
            }
            // --- 创建成功，刷新界面 ---
            this.mask = false;
            this.refresh();
        },
        // --- 刷新当前路径 ---
        refresh: async function(this: IVue, parent?: string, push?: boolean) {
            if (parent === undefined) {
                parent = globalConst.parent;
            }
            if (push === undefined) {
                push = true;
            }
            this.maskt = true;
            try {
                const r = await tool.post(globalConst.urlBase + 'api/zuny/getList', {
                    'parent': parent
                });
                const j = await r.json();
                if (j.result <= 0) {
                    this.maskt = false;
                    this.dialogType = 'dialog';
                    this.dialogTxt = j.msg;
                    return;
                }
                this.maskt = false;
                globalConst.parent = j.parent;
                this.path = j.path;
                this.pathKey = j.pathKey;
                this.list = j.list;
                // --- 添加浏览器记录 ---
                document.title = j.title + ' - Powered by Zuny';
                if (push) {
                    window.history.pushState({
                        'key': j.parent
                    }, j.title + ' - Powered by Zuny', globalConst.urlBase + (j.parent ? 'folder/' + j.parent : ''));
                }
            }
            catch {
                this.maskt = false;
                this.dialogType = 'dialog';
                this.dialogTxt = 'Network error.';
            }
        },
        // --- 进入目录 ajax ---
        go: async function(this: IVue, e: MouseEvent, key: string) {
            e.preventDefault();
            this.refresh(key);
        },
        // --- 打开登录 ---
        login: function(this: IVue) {
            this.hideMenu();
            if (this.account) {
                // --- 已登录 ---
                this.dialogType = 'logout';
                return;
            }
            this.dialogType = 'login';
            this.user = '';
            this.password = '';
            this.captcha = '';
            this.reCaptcha();
        },
        reCaptcha: function(this: IVue) {
            this.captchaImg = globalConst.urlBase + 'api/zuny/captcha?' + Math.random();
        },
        move: function(this: IVue) {
            this.moveRefresh();
            this.dialogType = 'move';
            this.hideMenu();
        },
        toMove: async function(this: IVue) {
            this.mask = true;
            this.dialogType = '';
            const r = await tool.post(globalConst.urlBase + 'api/zuny/move', {
                'key': this.skey,
                'type': this.stype,
                'to': this.mpathKey.length ? this.mpathKey[this.mpathKey.length - 1] : '',
                '_xsrf': globalConst.xsrf
            });
            const j = await r.json();
            if (j.result <= 0) {
                this.mask = false;
                this.dialogType = 'dialog';
                this.dialogTxt = j.msg;
                return;
            }
            this.mask = false;
            this.refresh();
        },
        // --- 刷新移动目录的当前路径 ---
        moveRefresh: async function(this: IVue, parent: string = '') {
            this.maskt = true;
            try {
                const r = await tool.post(globalConst.urlBase + 'api/zuny/getList', {
                    'parent': parent,
                    'file': '0'
                });
                const j = await r.json();
                if (j.result <= 0) {
                    this.maskt = false;
                    this.dialogType = 'dialog';
                    this.dialogTxt = j.msg;
                    return;
                }
                this.maskt = false;
                this.mpath = j.path;
                this.mpathKey = j.pathKey;
                this.mlist = j.list;
            }
            catch {
                this.maskt = false;
                this.dialogType = 'dialog';
                this.dialogTxt = 'Network error.';
            }
        },
        toLogin: async function(this: IVue) {
            this.mask = true;
            this.dialogType = '';
            const r = await tool.post(globalConst.urlBase + 'api/zuny/login', {
                'name': this.user,
                'password': this.password,
                'captcha': this.captcha,
                '_xsrf': globalConst.xsrf
            });
            const j = await r.json();
            if (j.result <= 0) {
                this.mask = false;
                this.dialogType = 'dialog';
                this.dialogTxt = j.msg;
                return;
            }
            this.mask = false;
            this.account = j.account;
        },
        toLogout: async function(this: IVue) {
            this.mask = true;
            this.dialogType = '';
            await fetch(globalConst.urlBase + 'api/zuny/logout');
            this.mask = false;
            this.account = false;
        },
        // --- 打开 item 菜单 ---
        openItemMenu: async function(this: IVue, el: HTMLElement, type: 'folder' | 'file', name: string, note: string, key: string) {
            if (this.itemMenuShow) {
                return;
            }
            if (this.account === '') {
                this.mask = true;
                const r = await fetch(globalConst.urlBase + 'api/zuny/checkLogin');
                const j = await r.json();
                this.account = j.account;
                this.mask = false;
            }
            if (!this.account) {
                // --- 没登录要先登录 ---
                this.login();
                return;
            }
            this.stype = type;
            this.skey = key;
            this.folderName = name;
            this.folderNote = note;
            const rect = el.getBoundingClientRect();
            this.$refs.itemMenu.style.left = rect.x + 'px';
            this.$refs.itemMenu.style.top = (rect.y + rect.height + 10) + 'px'
            this.itemMenuShow = true;
        },
        rename: function(this: IVue) {
            this.dialogType = 'rename';
            this.hideMenu();
        },
        toRename: async function(this: IVue) {
            this.mask = true;
            this.dialogType = '';
            const r = await tool.post(globalConst.urlBase + 'api/zuny/rename', {
                'key': this.skey,
                'name': this.folderName,
                'note': this.folderNote,
                'type': this.stype,
                '_xsrf': globalConst.xsrf
            });
            const j = await r.json();
            if (j.result <= 0) {
                this.mask = false;
                this.dialogType = 'dialog';
                this.dialogTxt = j.msg;
                return;
            }
            // --- 重命名成功，刷新界面 ---
            this.mask = false;
            this.refresh();
        },
        remove: function(this: IVue) {
            this.dialogType = 'remove';
            this.hideMenu();
        },
        toRemove: async function(this: IVue) {
            this.mask = true;
            this.dialogType = '';
            const r = await tool.post(globalConst.urlBase + 'api/zuny/remove', {
                'key': this.skey,
                'type': this.stype,
                '_xsrf': globalConst.xsrf
            });
            const j = await r.json();
            if (j.result <= 0) {
                this.mask = false;
                this.dialogType = 'dialog';
                this.dialogTxt = j.msg;
                return;
            }
            // --- 删除成功，刷新界面 ---
            this.mask = false;
            this.refresh();
        }
    },
    mounted: async function(this: IVue): Promise<void> {
        this.list = globalConst.list;
        this.path = globalConst.path;
        this.pathKey = globalConst.pathKey;
        vroot = this;
        if (!document.body.innerHTML.includes('https://www.maiyun.net/zuny')) {
            document.body.insertAdjacentHTML('beforeend', '<div style="position:fixed;font-size:12px;right:0;bottom:0;color:rgba(0,0,0,.4);">P'+'owered by <a href="https://www.maiyun.net" target="_blank">MAIYUN.NET</a> <a href="https://www.maiyun.net/zuny" target="_blank">Zuny</a></div>');
        }
        await this.$nextTick();
        // --- menu 处理 mousedown ---
        const menus: NodeListOf<HTMLElement> = document.querySelectorAll('.menu');
        for (const menu of menus) {
            menu.addEventListener('mousedown', (e) => {
                e.stopPropagation();
            });
        }
    }
});
const ls = document.querySelectorAll('.removed');
for (const item of ls) {
    item.remove();
}
app.mount('#panel');

document.addEventListener('mousedown', function() {
    vroot.hideMenu();
});
window.addEventListener('popstate', function(e) {
    vroot.refresh(e.state?.key ?? '', false);
});
